import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/*

Modification History
--------------------

Version 1.0
	This example shows how the elements of XML data are read into a program

Version 2.0
	 This example shows how the errors in XML data are trapped
     Note that the application gets the callbacks till the parsing error is found

Version 3.0

	This example demonstrates how attributes are read in
	Two access mechanisms demonstrated
	1.by attribute index;
    2.by qualified (prefixed) name.

Version 4.0
	This example demonstrates how PIs are handled

	Note: The first line in the CarParts.xml is a PI (XML document declaration)
	that identifies XML document and its encoding scheme.

	The content is all after the first XML document declaration.

*/


public class MyXMLHandler extends DefaultHandler {


	public void startDocument()
    {
        System.out.println("\n Start Document: -----Reading the document CarParts.xml with MyXMLHandler------\n");
    }

    public void startElement(String namespaceURI, String localName,
                             String qualifiedName, Attributes elementAttributes)
    {
        System.out.println("Start Element-> "+qualifiedName);

		/* Get the index of price attributes*/
		int indexOfPrice=elementAttributes.getIndex("price");

		/* If a price attribute does not exist then all the attributes are printed out*/
		/* Note that in the sample the attribute name is case sensitive so all attributes
			of carstereo get printed out*/
		if(-1==indexOfPrice)
			printAllAttributes(elementAttributes);
		else
			System.out.println("\tPrice= "+elementAttributes.getValue("price"));
    }

    public void endElement(String namespaceURI, String localName,
                             String qualifiedName)
    {
        System.out.println("End Element-> "+qualifiedName);
    }



	public void processingInstruction(java.lang.String target,
	                                  java.lang.String data)
	{
		System.out.println("Processing Instruction-> on target: "+target+ "\n\t\t\t and data: "+data);
	}


    public void endDocument()
    {
        System.out.println("\n End Document: ----------------Finished Reading the document---------------------\n");
    }



	public void printAllAttributes(Attributes elementAttributes)
	{
		System.out.println("\tTotal Number of Attributes: "+elementAttributes.getLength());
		for(int i=0;i<elementAttributes.getLength();i++)
		{
			System.out.println("\t\tAttribute: "+elementAttributes.getQName(i)+
										    " = "+elementAttributes.getValue(i));
		}
	}



    static public void main(String[] args) throws Exception {

        System.out.println("\nVersion 4.0 of MyXMLHandler in example04\n");

		/*Create a SAX Parser Factory*/
        SAXParserFactory parseFactory = SAXParserFactory.newInstance();

		/*Obtain a SAX Parser */
        SAXParser saxParser = parseFactory.newSAXParser();

		/*XML Reader is the interface for reading an XML document using callbacks*/
        XMLReader xmlReader = saxParser.getXMLReader();

		/*set the error handler*/
		xmlReader.setErrorHandler(new MyErrorHandler());

		/*Attach ContentHandler - the callbacks like startDocument,startElement etc. are o
		  overridden by the setContentHandler to trap them into user code*/
        xmlReader.setContentHandler(new MyXMLHandler());

		try{
			/*Parse an XML document - the document is read and overriden callbacks in the MyXMLHandler are invoked*/
	        xmlReader.parse("CarParts.xml");
		}
		catch (SAXParseException saxException) {
					/* If there are errors in XML data are trapped and location is displayed*/
					System.out.println("\n\nError in data.xml at line:"+saxException.getLineNumber()+"("+saxException.getColumnNumber()+")\n");
					System.out.println(saxException.toString());
				}
		catch (SAXException saxEx) {
					/* If there are errors in XML data, the detailed message of the exception is displayed*/
					System.out.println(saxEx.getMessage());
				}
    }

	static class MyErrorHandler implements ErrorHandler
	{

        public void fatalError(SAXParseException saxException)
		{
            System.out.println("Fatal Error occurred "+ saxException);
        }

        public void error(SAXParseException saxException)
		{
            System.out.println("Error occurred "+ saxException);
        }

        public void warning(SAXParseException saxException)
		{
            System.out.println("warning occurred "+ saxException);
        }
    }
}
