
/*

Modification History
--------------------

Version 1.0
	This example shows how an XML file is read into a DOM

Version 2.0
	This example shows how the DTD informattion is got from DOM

Version 3.0
	This example shows how to access element nodes and determine their types

Version 4.0
	This example shows how to access an element randomly and determine its attributes and values

Version 5.0
	This example shows how to access the text node for an element node

Version 6.0
	This example shows how to access Entities and Processing Instructions

Version 7.0
	This example shows how to create an element

*/

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import java.io.File;
import java.io.IOException;
import org.w3c.dom.*;
import org.xml.sax.ErrorHandler;

public class MyDOMHandler {

   static Document document;

//   static String nodeTypes[] = {"ATTRIBUTE_NODE","CDATA_SECTION_NODE ","COMMENT_NODE","DOCUMENT_FRAGMENT_NODE ","DOCUMENT_NODE","DOCUMENT_TYPE_NODE","ELEMENT_NODE","ENTITY_NODE","ENTITY_REFERENCE_NODE","NOTATION_NODE","PROCESSING_INSTRUCTION_NODE","TEXT_NODE"};
	static String  nodeTypes[] ={
								"none",
								"Element",
								"Attr",
								"Text",
								"CDATA",
								"EntityRef",
								"Entity",
								"ProcInstr",
								"Comment",
								"Document",
								"DocType",
								"DocFragment",
								"Notation",
								};

   static int nodeLevel=0;

   public static void printDomNode(Node node)
   {
		for(int j=0;j<nodeLevel;j++)
			System.out.print("\t");
	  	System.out.println(node.getNodeName() + "\" [" + nodeTypes[node.getNodeType()]+"]");

	  	NamedNodeMap nodeMap = node.getAttributes();
		for (int j = 0; null!= nodeMap && j < nodeMap.getLength(); j++)
		{
			Node attributeNode = nodeMap.item(j);
			for(int k=0;k<nodeLevel;k++)
				System.out.print("\t");

			System.out.println("Attr Name: "+attributeNode.getNodeName() +" Value: "+attributeNode.getNodeValue());
		}


		NodeList list = node.getChildNodes();
		nodeLevel++;
		for(int i=0;i<list.getLength();i++)
		{
			printDomNode(list.item(i));
		}
		nodeLevel--;
   }


   public static void main(String argv[])
   {

    	NodeList list;
		Node elementNode;
		int ListLength;
		DocumentBuilderFactory factory =
      DocumentBuilderFactory.newInstance();
		factory.setValidating(true);

      try {


		  DocumentBuilder builder = factory.newDocumentBuilder();
		  builder.setErrorHandler(new MyErrorHandler());
	      System.out.println("\n----------------- Creating the DOM from the CarParts.xml File ---------- \n");
		  	      document = builder.parse( new File("CarParts.xml") );
	      System.out.println("\n----------------- DOM Created from the CarParts.xml file ---------- \n");

	      // Getting the DTD Information
	      DocumentType docType = document.getDoctype();
	      System.out.println("\n-----------The Name of of the DTD is : " + docType.getName() + "\n");
	      System.out.println("\n-----------The public ID of the doctype is : " + docType.getSystemId() + "\n");

		 // Getting the root element
		  Element element = document.getDocumentElement();
		  System.out.println("\n---- The tag name of the document element is : " + element.getTagName() + "\n");

		 //Getting all nodes
/*		  list = document.getElementsByTagName("*");
		  ListLength = list.getLength();
		  System.out.println("\n---- The number of element nodes are : " + ListLength + "\n");

		  for (int i = 0; i < ListLength; i++)
		  {
			  	elementNode = list.item(i);
			  	System.out.println("The name of the node is : \"" + elementNode.getNodeName() + "\" and is of type : " + nodeTypes[elementNode.getNodeType()]);

		  }*/

		  printDomNode(document.getDocumentElement());

		  // Randomly accessing a element node, its attributes and values, for example engine

				 list = document.getElementsByTagName("engine");
				 ListLength = list.getLength();


		//Creating elements
					list = document.getElementsByTagName("engines");
					Element newNode = document.createElement("engine");
					System.out.println("List size is "+list.getLength()+" "+nodeTypes[newNode.getNodeType()] +" "+nodeTypes[list.item(0).getNodeType()]);

					//Attr newNodeAttributeId = document.createAttribute("id");
					//newNodeAttributeId.setValue("ID7232");
					//Attr newNodeAttributeType = document.createAttribute("Type");
					//Attr newNodeAttributeCapacity = document.createAttribute("Capacity");
					//Attr newNodeAttributePrice = document.createAttribute("Price");
					//newNode.setAttributeNode(newNodeAttributeId);
					newNode.setAttribute("id","ID7232");
					newNode.setAttribute("Type","Alfa-D");
					newNode.setAttribute("Capacity","1500CC");
					newNode.setAttribute("Price","USD1000");
					Node newNodeText = document.createTextNode("Engine 3");
					newNode.appendChild(newNodeText);
					((Element)list.item(0)).appendChild(newNode);

		//			 Node lastNode = list.item(ListLength - 1);
				 //lastNode.insertBefore(newNode, null);


	  printDomNode(document.getDocumentElement());
	  				list = document.getElementsByTagName("carstereos");
	  				Node tobeRmvd = list.item(0).getFirstChild().getNextSibling();
	  							((Element)list.item(0)).removeChild(tobeRmvd);
	  							document.getDocumentElement().normalize();
 printDomNode(document.getDocumentElement());

/*				 System.out.println("\n---- The number of engine element nodes are : " + ListLength + "\n");

				 for (int i = 0; i < ListLength; i++)
				 {
					  	elementNode = list.item(i);
					  	System.out.println("The name of the node is : " + elementNode.getNodeName() + "\n");
					  	NamedNodeMap nodeMap = elementNode.getAttributes();
						for (int j = 0; j < nodeMap.getLength(); j++)
						{
							Node attributeNode = nodeMap.item(j);
							System.out.println("Attribute Name: "+attributeNode.getNodeName() + "\n");
							System.out.println("Attribute Value: "+attributeNode.getNodeValue() + "\n");
						}

						//Getting the text node values
						Node textNode = elementNode.getLastChild(); //Since text node is the last child node of the element node
						System.out.println("The value of the text node is :" + textNode.getNodeValue().trim() + "\n");
				 }

*/

	  } catch (SAXParseException saxException) {
	  					/* If there are errors in XML data are trapped and location is displayed*/
	  					System.out.println("\n\nError in CarParts.xml at line:"+saxException.getLineNumber()+"("+saxException.getColumnNumber()+")\n");
	  					System.out.println(saxException.toString());
	  				}
	  		catch (SAXException saxEx) {
	  							/* If there are errors in XML data, the detailed message of the exception is displayed*/
	  							System.out.println(saxEx.getMessage());
	  			}

	  		catch (ParserConfigurationException pce) {
	  		      // Parser with specified options can't be built
	  		      pce.printStackTrace();
	  			}
	  		catch (IOException ioe) {
	  		      // I/O error
	        ioe.printStackTrace();
	  		}




	     }// End of main

	     static class MyErrorHandler implements ErrorHandler
	     	{

	             public void fatalError(SAXParseException saxException)
	     		{
	                 System.out.println("Fatal Error occurred "+ saxException);

	             }

	             public void error(SAXParseException saxException)
	     		{
	                 System.out.println("Error occurred "+ saxException);
	             }

	             public void warning(SAXParseException saxException)
	     		{
	                 System.out.println("warning occurred "+ saxException);
	             }
	      }
	  } // End of MyDOMHandler
