
/*

Modification History
--------------------

Version 1.0
	This example shows how an XML file is read into a DOM, manipulated and then transformed out into a XML document



*/

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import java.io.File;
import java.io.IOException;
import org.w3c.dom.*;
import org.xml.sax.ErrorHandler;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.OutputKeys;

public class MyDOMTransformer {

   static Document document;



   public static void main(String argv[])
   {

		DocumentBuilderFactory factory =
      DocumentBuilderFactory.newInstance();
		factory.setValidating(true);

      try {


		  DocumentBuilder builder = factory.newDocumentBuilder();
		  builder.setErrorHandler(new MyErrorHandler());
	      System.out.println("\n----------------- Creating the DOM from the CarParts.xml File ---------- \n");
		  	      document = builder.parse( new File("CarParts.xml") );
	      System.out.println("\n----------------- DOM Created from the CarParts.xml file ---------- \n");

	      //Create a transformer

	      TransformerFactory tnsfctry = TransformerFactory.newInstance();

	      Transformer transformer = tnsfctry.newTransformer();


	      //manipulating the dom to create a new engine element and remove the carstereo element

			NodeList list = document.getElementsByTagName("engines");
			Element newNode = document.createElement("engine");
			((Element)list.item(0)).appendChild(newNode);
			newNode.setAttribute("id","ID7232");
			newNode.setAttribute("type","Alfa-D");
			newNode.setAttribute("capacity","1500CC");
			newNode.setAttribute("price","USD1000");

			newNode.appendChild(document.createTextNode("Engine 3"));
			newNode.appendChild(document.createTextNode(""));
			newNode.appendChild(document.createTextNode(" contd"));

			list = document.getElementsByTagName("carstereos");
			Node tobeRmvd = list.item(0).getFirstChild().getNextSibling();
			((Element)list.item(0)).removeChild(tobeRmvd);

			document.getDocumentElement().normalize();

			String docTypeID = document.getDoctype().getSystemId();
			transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, docTypeID);
			DOMSource source = new DOMSource(document);
			StreamResult result = new StreamResult(System.out);
			transformer.transform(source,result);


	  } catch (SAXParseException saxException) {
	  					/* If there are errors in XML data are trapped and location is displayed*/
	  					System.out.println("\n\nError in CarParts.xml at line:"+saxException.getLineNumber()+"("+saxException.getColumnNumber()+")\n");
	  					System.out.println(saxException.toString());
	  				}
	  		catch (SAXException saxEx) {
	  							/* If there are errors in XML data, the detailed message of the exception is displayed*/
	  							System.out.println(saxEx.getMessage());
	  			}

	  		catch (ParserConfigurationException pce) {
	  		      // Parser with specified options can't be built
	  		      pce.printStackTrace();
	  			}
	  		catch (IOException ioe) {
	  		      // I/O error
	        ioe.printStackTrace();
	  		}
			catch (TransformerConfigurationException tcex)
			{
				System.out.println(tcex.getMessageAndLocation());
			}
			catch (TransformerException te)
			{
				System.out.println(te.getMessageAndLocation());
		}



	     }// End of main

	     static class MyErrorHandler implements ErrorHandler
	     	{

	             public void fatalError(SAXParseException saxException)
	     		{
	                 System.out.println("Fatal Error occurred "+ saxException);

	             }

	             public void error(SAXParseException saxException)
	     		{
	                 System.out.println("Error occurred "+ saxException);
	             }

	             public void warning(SAXParseException saxException)
	     		{
	                 System.out.println("warning occurred "+ saxException);
	             }
	      }
	  } // End of MyDOMHandler
