
import javax.xml.registry.*;
import javax.xml.registry.infomodel.*;
import java.net.*;
import java.util.*;


public class MyRegistryQuery {
    Connection connection = null;

    public MyRegistryQuery() {}

    public static void main(String[] args) {

        if (args.length < 1) {
            System.out.println("Usage: runclient " +
                "MyRegistryQuery string");
            System.exit(1);
        }
        String queryString = new String(args[0]);
        System.out.println("Query string is " + queryString);

        MyRegistryQuery mrq = new MyRegistryQuery();

       mrq.connect();

       mrq.runQuery(queryString);
    }


    public void connect()
    {


         String queryURL =
		             "http://localhost:8080/registry-server/RegistryServerServlet";
		         String publishURL =
		             "http://localhost:8080/registry-server/RegistryServerServlet";

        String httpProxyHost = "";
        String httpProxyPort = "8080";


        Properties props = new Properties();
        props.setProperty("javax.xml.registry.queryManagerURL",
            queryURL);
        props.setProperty("com.sun.xml.registry.http.proxyHost",
            httpProxyHost);
        props.setProperty("com.sun.xml.registry.http.proxyPort",
            httpProxyPort);

        try {
            // Create the connection,
            ConnectionFactory factory =
                ConnectionFactory.newInstance();
            factory.setProperties(props);
            connection = factory.createConnection();
            System.out.println("Connection created to the registry");
        } catch (Exception e) {
            e.printStackTrace();

        }
    }


    public void runQuery(String qString) {
        RegistryService rsvrc = null;
        BusinessQueryManager bsnsqrymgr = null;

        try {
            // Get registry service and query manager
            rsvrc = connection.getRegistryService();
            bsnsqrymgr = rsvrc.getBusinessQueryManager();
            System.out.println("Got registry service and " +
                "query manager");

            // Specify the find qualifiers and name patterns
            Collection findQualifiers = new ArrayList();
            findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
            Collection namePatterns = new ArrayList();
            namePatterns.add("%" + qString + "%");

            // Find organization that match the name pattern containing the query string
            BulkResponse response =
                bsnsqrymgr.findOrganizations(findQualifiers,
                    namePatterns, null, null, null, null);
            Collection orgs = response.getCollection();

            // Display information about the organizations found
            Iterator iter = orgs.iterator();
            if (!(iter.hasNext())) {
                System.out.println("There are no organizations which match the supplied pattern");
            } else while (iter.hasNext()) {
                Organization orgn = (Organization) iter.next();
                //System.out.println("Organization name: " + getName(orgn));
                System.out.println("Organization name: " + orgn.getName().getValue());
                System.out.println("Organization description: " +
                   orgn.getDescription().getValue());
                System.out.println("Organization key id: " + orgn.getKey().getId());

                // Display contact information such as contact person's name, phone number etc
                User prmrycont = orgn.getPrimaryContact();
                if (prmrycont != null) {
                    PersonName prmrycontName = prmrycont.getPersonName();
                    System.out.println(" Contact name: " +
                        prmrycontName.getFullName());
                    Collection phoneNums =
                        prmrycont.getTelephoneNumbers(null);
                   Iterator phoneiter = phoneNums.iterator();
                    while (phoneiter.hasNext()) {
                        TelephoneNumber number =
                            (TelephoneNumber) phoneiter.next();
                        System.out.println("  Phone number: " +
                            number.getNumber());
                    }
                    Collection emailadd = prmrycont.getEmailAddresses();
                    Iterator emailiter = emailadd.iterator();
                    while (emailiter.hasNext()) {
                        EmailAddress eAd =
                            (EmailAddress) emailiter.next();
                        System.out.println("  Email Address: " +
                            eAd.getAddress());
                    }
                }
                // Display service and binding information
                Collection services = orgn.getServices();
                Iterator srvciter = services.iterator();
                while (srvciter.hasNext()) {
                    Service srvc = (Service) srvciter.next();
                    System.out.println(" Service name: " +
                        srvc.getName().getValue());
                    System.out.println(" Service description: " +
                        srvc.getDescription().getValue());
                    Collection serviceBindings =
                        srvc.getServiceBindings();
                    Iterator srvcbndgiter = serviceBindings.iterator();
                    while (srvcbndgiter.hasNext()) {
                        ServiceBinding sb =
                            (ServiceBinding) srvcbndgiter.next();
                        System.out.println("  Binding " +
                            "Description: " +
                            sb.getDescription().getValue());
                        System.out.println("  Access URI: " +
                            sb.getAccessURI());
                    }
                }
                // Print spacer between organizations
                System.out.println(" --- ");
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally  {
            // At end, close connection to registry
            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {}
            }
        }
    }



}
